<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPaybywallet extends PayplansAppPayment
{	
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		$utilizeWallet = XiFactory::getConfig()->walletUtilizationAllowed;
		
		if($utilizeWallet){
			return parent::isApplicable($refObject, $eventName);
		}
		
		return false;
	}
		
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice 	   = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  	   = $invoice->getTotal();
		$walletBalance = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE)->getWalletBalance();
			
		$this->assign('amount', $amount);
		$this->assign('invoice', $invoice);
		$this->assign('payment_key', $payment->getKey());
		$this->assign('wallet_balance', $walletBalance);
		$this->assign('posturl',	XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey()));
        return $this->_render('form');
	}

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel')
		   return ;

		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount	  = $invoice->getTotal();

		$errors = $this->_checkPaymentRequest($payment, $amount);
				
		$errorLog = PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($errorLog);			
		if($record && !empty($record)){
			$action = 'error';
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
	
		if($action == 'success'){
			$recurrence_count = $this->_getRecurrenceCount($invoice);
			
			$data['transaction'] = array('amount'=>$amount);
			$this->_processTransaction($payment, $data, $recurrence_count-1);
									
			return true;
		}
	}
	
	//before subscription expiry process the future payment if allowed
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring() && (XiFactory::getConfig()->walletForRecurringAllowed))
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

			if($recurrence_count > 0)
			{
				$amount = $invoice->getTotal($invoice_count);
				$errors = $this->_checkPaymentRequest($payment, $amount);
				//if any error exists then do not create transaction
				if(!empty($errors)){
					return true;
				}
				
				$pendingRecurrence = $this->_getRecurrenceCount($invoice) - $invoice_count;
				$data = array();
				$data['transaction'] = array('amount'=>$amount);
				$this->_processTransaction($payment, $data, $pendingRecurrence);

				return true;
			}
		}
	}
	

	protected function _checkPaymentRequest(PayplansPayment $payment, $amount)
	{
		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$user     = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$walletBalance = $user->getWalletBalance();
		
		if(defined('PAYPLANS_UNIT_TEST_MODE')===true){
				return true;
			}
		$errors = array();
		//when wallet utilization not allowed
		if(!XiFactory::getConfig()->walletUtilizationAllowed){
			$errors[] = XiText::_('COM_PAYPLANS_PAY_BY_WALLET_WALLET_UTILIZATION_NO_ALLOWED');
		}
		
		//	amount in wallet is insufficient for the invoice
		if(floatval($amount) > floatval($walletBalance)){
			$errors[] = XiText::_('COM_PAYPLANS_PAY_BY_WALLET_INSUFFICIENT_BALANCE_IN_WALLET');
		}

		//recurring payments are not allowed to utilize wallet balance
		if($invoice->isRecurring() && (!XiFactory::getConfig()->walletForRecurringAllowed)){
			$errors[] = XiText::_('COM_PAYPLANS_PAY_BY_WALLET_RECURRING_SUBSCRIPTION_CAN_NOT_UTILIZE_WALLET_BALANCE');
		}

		if(!empty($errors)){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_PAY_BY_WALLET_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors);
		}
		
		return $errors;
	}
	
	protected function _processTransaction(PayplansPayment $payment, $data, $pendingRecurrence=null)
	{
		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount   = $data['transaction']['amount'];
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', 0)
					->set('gateway_txn_id', 0)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($data['transaction']))
					->set('message', 'COM_PAYPLANS_APP_PAY_BY_WALLET_TRANSACTION_CREATED_FOR_INVOICE')
					->save();

		//update recurrence count in payment gateway params
		if($invoice->isRecurring()){
			$params = array('pending_recur_count' => $pendingRecurrence);
	
			$payment->set('gateway_params', PayplansHelperParam::arrayToIni($params))
						->save();
		}
		
		$args = array($transaction, $amount);
		PayplansHelperEvent::trigger('onPayplansWalletUpdate', $args);
		
		return true;
	}
	
	protected function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		// for lifetime recurrence
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		parent::onPayplansPaymentTerminate($payment, $controller);
		return $this->_render('cancel');
	}
}
